/*
 * This file is part of ViaFabricPlus - https://github.com/FlorianMichael/ViaFabricPlus
 * Copyright (C) 2021-2023 FlorianMichael/EnZaXD
 * Copyright (C) 2023      RK_01/RaphiMC and contributors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.florianmichael.viafabricplus.fixes.data;

import de.florianmichael.viafabricplus.injection.access.IClientConnection;
import de.florianmichael.viafabricplus.protocolhack.ProtocolHack;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.network.ClientPlayNetworkHandler;
import net.minecraft.item.Item;
import net.raphimc.vialegacy.protocols.classic.protocolc0_28_30toc0_28_30cpe.data.ClassicProtocolExtension;
import net.raphimc.vialegacy.protocols.classic.protocolc0_28_30toc0_28_30cpe.storage.ExtensionProtocolMetadataStorage;
import net.raphimc.vialoader.util.VersionEnum;
import net.raphimc.vialoader.util.VersionRange;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static net.minecraft.item.Items.*;
import static net.raphimc.vialoader.util.VersionEnum.*;
import static net.raphimc.vialoader.util.VersionRange.*;

public class ItemRegistryDiff {

    public static final Map<Item, VersionRange> ITEM_DIFF = new HashMap<>();
    public static final List<Item> EXTENDED_CLASSIC_ITEMS = new ArrayList<>();

    static {
        ITEM_DIFF.put(CHERRY_LOG, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_WOOD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(STRIPPED_CHERRY_LOG, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(STRIPPED_CHERRY_WOOD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_PLANKS, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_STAIRS, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_SLAB, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_FENCE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_FENCE_GATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_DOOR, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_TRAPDOOR, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_PRESSURE_PLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_BUTTON, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_BLOCK, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(STRIPPED_BAMBOO_BLOCK, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_PLANKS, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_STAIRS, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_SLAB, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_FENCE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_FENCE_GATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_DOOR, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_TRAPDOOR, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_PRESSURE_PLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_BUTTON, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_MOSAIC, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_MOSAIC_STAIRS, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_MOSAIC_SLAB, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_BOAT, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_CHEST_BOAT, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_RAFT, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_CHEST_RAFT, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(MUSIC_DISC_RELIC, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SNIFFER_SPAWN_EGG, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CAMEL_SPAWN_EGG, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CALIBRATED_SCULK_SENSOR, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHISELED_BOOKSHELF, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(DECORATED_POT, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SUSPICIOUS_SAND, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SUSPICIOUS_GRAVEL, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(PIGLIN_HEAD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(OAK_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SPRUCE_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BIRCH_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(JUNGLE_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(ACACIA_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(DARK_OAK_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(MANGROVE_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CRIMSON_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(WARPED_HANGING_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_LEAVES, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_SAPLING, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(CHERRY_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BAMBOO_SIGN, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(TORCHFLOWER, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(TORCHFLOWER_SEEDS, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(PINK_PETALS, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(PITCHER_PLANT, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(PITCHER_POD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SNIFFER_EGG, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BRUSH, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(ANGLER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(ARCHER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(ARMS_UP_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BLADE_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BREWER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(BURN_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(DANGER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(EXPLORER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(FRIEND_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(HEART_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(HEARTBREAK_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(HOWL_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(MINER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(MOURNER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(PLENTY_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(PRIZE_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SHEAF_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SHELTER_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SKULL_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SNORT_POTTERY_SHERD, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(NETHERITE_UPGRADE_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SENTRY_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(DUNE_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(COAST_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(WILD_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(WARD_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(EYE_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(VEX_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(TIDE_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SNOUT_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(RIB_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SPIRE_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(WAYFINDER_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SHAPER_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(SILENCE_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(RAISER_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));
        ITEM_DIFF.put(HOST_ARMOR_TRIM_SMITHING_TEMPLATE, andNewer(r1_20tor1_20_1));

        ITEM_DIFF.put(IRON_GOLEM_SPAWN_EGG, andNewer(r1_19_3));
        ITEM_DIFF.put(SNOW_GOLEM_SPAWN_EGG, andNewer(r1_19_3));
        ITEM_DIFF.put(WITHER_SPAWN_EGG, andNewer(r1_19_3));
        ITEM_DIFF.put(ENDER_DRAGON_SPAWN_EGG, andNewer(r1_19_3));
        ITEM_DIFF.put(SPAWNER, andNewer(r1_19_3));

        ITEM_DIFF.put(OCHRE_FROGLIGHT, andNewer(r1_19));
        ITEM_DIFF.put(PEARLESCENT_FROGLIGHT, andNewer(r1_19));
        ITEM_DIFF.put(VERDANT_FROGLIGHT, andNewer(r1_19));
        ITEM_DIFF.put(FROGSPAWN, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_LEAVES, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_LOG, andNewer(r1_19));
        ITEM_DIFF.put(STRIPPED_MANGROVE_LOG, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_PLANKS, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_PROPAGULE, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_ROOTS, andNewer(r1_19));
        ITEM_DIFF.put(MUDDY_MANGROVE_ROOTS, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_WOOD, andNewer(r1_19));
        ITEM_DIFF.put(STRIPPED_MANGROVE_WOOD, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_CHEST_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_PRESSURE_PLATE, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_TRAPDOOR, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_FENCE_GATE, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_DOOR, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_FENCE, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_SIGN, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_SLAB, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_STAIRS, andNewer(r1_19));
        ITEM_DIFF.put(MANGROVE_BUTTON, andNewer(r1_19));
        ITEM_DIFF.put(MUD, andNewer(r1_19));
        ITEM_DIFF.put(MUD_BRICKS, andNewer(r1_19));
        ITEM_DIFF.put(MUD_BRICK_WALL, andNewer(r1_19));
        ITEM_DIFF.put(MUD_BRICK_SLAB, andNewer(r1_19));
        ITEM_DIFF.put(MUD_BRICK_STAIRS, andNewer(r1_19));
        ITEM_DIFF.put(PACKED_MUD, andNewer(r1_19));
        ITEM_DIFF.put(REINFORCED_DEEPSLATE, andNewer(r1_19));
        ITEM_DIFF.put(SCULK, andNewer(r1_19));
        ITEM_DIFF.put(SCULK_CATALYST, andNewer(r1_19));
        ITEM_DIFF.put(SCULK_SHRIEKER, andNewer(r1_19));
        ITEM_DIFF.put(SCULK_VEIN, andNewer(r1_19));
        ITEM_DIFF.put(TADPOLE_BUCKET, andNewer(r1_19));
        ITEM_DIFF.put(DISC_FRAGMENT_5, andNewer(r1_19));
        ITEM_DIFF.put(ECHO_SHARD, andNewer(r1_19));
        ITEM_DIFF.put(GOAT_HORN, andNewer(r1_19));
        ITEM_DIFF.put(MUSIC_DISC_5, andNewer(r1_19));
        ITEM_DIFF.put(RECOVERY_COMPASS, andNewer(r1_19));
        ITEM_DIFF.put(OAK_CHEST_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(SPRUCE_CHEST_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(BIRCH_CHEST_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(JUNGLE_CHEST_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(ACACIA_CHEST_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(DARK_OAK_CHEST_BOAT, andNewer(r1_19));
        ITEM_DIFF.put(ALLAY_SPAWN_EGG, andNewer(r1_19));
        ITEM_DIFF.put(FROG_SPAWN_EGG, andNewer(r1_19));
        ITEM_DIFF.put(TADPOLE_SPAWN_EGG, andNewer(r1_19));
        ITEM_DIFF.put(WARDEN_SPAWN_EGG, andNewer(r1_19));

        ITEM_DIFF.put(MUSIC_DISC_OTHERSIDE, andNewer(r1_18tor1_18_1));

        ITEM_DIFF.put(CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(WHITE_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(ORANGE_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(MAGENTA_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(LIGHT_BLUE_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(YELLOW_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(LIME_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(PINK_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(GRAY_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(LIGHT_GRAY_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(CYAN_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(PURPLE_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(BLUE_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(BROWN_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(GREEN_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(RED_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(BLACK_CANDLE, andNewer(r1_17));
        ITEM_DIFF.put(AMETHYST_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(BUDDING_AMETHYST, andNewer(r1_17));
        ITEM_DIFF.put(AMETHYST_CLUSTER, andNewer(r1_17));
        ITEM_DIFF.put(LARGE_AMETHYST_BUD, andNewer(r1_17));
        ITEM_DIFF.put(MEDIUM_AMETHYST_BUD, andNewer(r1_17));
        ITEM_DIFF.put(SMALL_AMETHYST_BUD, andNewer(r1_17));
        ITEM_DIFF.put(TUFF, andNewer(r1_17));
        ITEM_DIFF.put(CALCITE, andNewer(r1_17));
        ITEM_DIFF.put(TINTED_GLASS, andNewer(r1_17));
        ITEM_DIFF.put(SCULK_SENSOR, andNewer(r1_17));
        ITEM_DIFF.put(WEATHERED_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(OXIDIZED_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(EXPOSED_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(COPPER_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(COPPER_ORE, andNewer(r1_17));
        ITEM_DIFF.put(WEATHERED_CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(OXIDIZED_CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(EXPOSED_CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WEATHERED_CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(OXIDIZED_CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(EXPOSED_CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(WEATHERED_CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(OXIDIZED_CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(EXPOSED_CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_COPPER_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_WEATHERED_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_EXPOSED_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_WEATHERED_CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_EXPOSED_CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_WEATHERED_CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_EXPOSED_CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_WEATHERED_CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_EXPOSED_CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(LIGHTNING_ROD, andNewer(r1_17));
        ITEM_DIFF.put(POINTED_DRIPSTONE, andNewer(r1_17));
        ITEM_DIFF.put(DRIPSTONE_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(GLOW_LICHEN, andNewer(r1_17));
        ITEM_DIFF.put(AZALEA_LEAVES, andNewer(r1_17));
        ITEM_DIFF.put(SPORE_BLOSSOM, andNewer(r1_17));
        ITEM_DIFF.put(AZALEA, andNewer(r1_17));
        ITEM_DIFF.put(FLOWERING_AZALEA, andNewer(r1_17));
        ITEM_DIFF.put(FLOWERING_AZALEA_LEAVES, andNewer(r1_17));
        ITEM_DIFF.put(MOSS_CARPET, andNewer(r1_17));
        ITEM_DIFF.put(MOSS_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(BIG_DRIPLEAF, andNewer(r1_17));
        ITEM_DIFF.put(SMALL_DRIPLEAF, andNewer(r1_17));
        ITEM_DIFF.put(ROOTED_DIRT, andNewer(r1_17));
        ITEM_DIFF.put(HANGING_ROOTS, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_GOLD_ORE, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_IRON_ORE, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_LAPIS_ORE, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_DIAMOND_ORE, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_REDSTONE_ORE, andNewer(r1_17));
        ITEM_DIFF.put(CHISELED_DEEPSLATE, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE, andNewer(r1_17));
        ITEM_DIFF.put(COBBLED_DEEPSLATE, andNewer(r1_17));
        ITEM_DIFF.put(COBBLED_DEEPSLATE_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(COBBLED_DEEPSLATE_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(COBBLED_DEEPSLATE_WALL, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_BRICK_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_BRICK_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_BRICK_WALL, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_BRICKS, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_TILE_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_TILE_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_TILE_WALL, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_TILES, andNewer(r1_17));
        ITEM_DIFF.put(POLISHED_DEEPSLATE, andNewer(r1_17));
        ITEM_DIFF.put(POLISHED_DEEPSLATE_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(POLISHED_DEEPSLATE_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(POLISHED_DEEPSLATE_WALL, andNewer(r1_17));
        ITEM_DIFF.put(SMOOTH_BASALT, andNewer(r1_17));
        ITEM_DIFF.put(CRACKED_DEEPSLATE_BRICKS, andNewer(r1_17));
        ITEM_DIFF.put(CRACKED_DEEPSLATE_TILES, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_COAL_ORE, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_COPPER_ORE, andNewer(r1_17));
        ITEM_DIFF.put(DEEPSLATE_EMERALD_ORE, andNewer(r1_17));
        ITEM_DIFF.put(INFESTED_DEEPSLATE, andNewer(r1_17));
        ITEM_DIFF.put(LIGHT, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_OXIDIZED_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_OXIDIZED_CUT_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_OXIDIZED_CUT_COPPER_SLAB, andNewer(r1_17));
        ITEM_DIFF.put(WAXED_OXIDIZED_CUT_COPPER_STAIRS, andNewer(r1_17));
        ITEM_DIFF.put(RAW_COPPER_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(RAW_GOLD_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(RAW_IRON_BLOCK, andNewer(r1_17));
        ITEM_DIFF.put(COPPER_INGOT, andNewer(r1_17));
        ITEM_DIFF.put(BUNDLE, andNewer(r1_17));
        ITEM_DIFF.put(AMETHYST_SHARD, andNewer(r1_17));
        ITEM_DIFF.put(SPYGLASS, andNewer(r1_17));
        ITEM_DIFF.put(POWDER_SNOW_BUCKET, andNewer(r1_17));
        ITEM_DIFF.put(AXOLOTL_BUCKET, andNewer(r1_17));
        ITEM_DIFF.put(GLOW_ITEM_FRAME, andNewer(r1_17));
        ITEM_DIFF.put(GLOW_INK_SAC, andNewer(r1_17));
        ITEM_DIFF.put(GLOW_BERRIES, andNewer(r1_17));
        ITEM_DIFF.put(RAW_IRON, andNewer(r1_17));
        ITEM_DIFF.put(RAW_GOLD, andNewer(r1_17));
        ITEM_DIFF.put(RAW_COPPER, andNewer(r1_17));
        ITEM_DIFF.put(AXOLOTL_SPAWN_EGG, andNewer(r1_17));
        ITEM_DIFF.put(GLOW_SQUID_SPAWN_EGG, andNewer(r1_17));
        ITEM_DIFF.put(GOAT_SPAWN_EGG, andNewer(r1_17));

        ITEM_DIFF.put(ANCIENT_DEBRIS, andNewer(r1_16));
        ITEM_DIFF.put(BASALT, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_BUTTON, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_BUTTON, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_DOOR, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_DOOR, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_FENCE, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_FENCE, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_FENCE_GATE, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_FENCE_GATE, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_FUNGUS, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_FUNGUS, andNewer(r1_16));
        ITEM_DIFF.put(NETHER_SPROUTS, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_NYLIUM, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_NYLIUM, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_PLANKS, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_PLANKS, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_PRESSURE_PLATE, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_PRESSURE_PLATE, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_ROOTS, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_ROOTS, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_SIGN, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_SIGN, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_SLAB, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_SLAB, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_STAIRS, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_STAIRS, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_STEM, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_STEM, andNewer(r1_16));
        ITEM_DIFF.put(STRIPPED_CRIMSON_STEM, andNewer(r1_16));
        ITEM_DIFF.put(STRIPPED_WARPED_STEM, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_TRAPDOOR, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_TRAPDOOR, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_BLOCK, andNewer(r1_16));
        ITEM_DIFF.put(SHROOMLIGHT, andNewer(r1_16));
        ITEM_DIFF.put(SOUL_LANTERN, andNewer(r1_16));
        ITEM_DIFF.put(SOUL_TORCH, andNewer(r1_16));
        ITEM_DIFF.put(SOUL_SOIL, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_WART_BLOCK, andNewer(r1_16));
        ITEM_DIFF.put(WEEPING_VINES, andNewer(r1_16));
        ITEM_DIFF.put(TARGET, andNewer(r1_16));
        ITEM_DIFF.put(CRYING_OBSIDIAN, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_HYPHAE, andNewer(r1_16));
        ITEM_DIFF.put(STRIPPED_WARPED_HYPHAE, andNewer(r1_16));
        ITEM_DIFF.put(CRIMSON_HYPHAE, andNewer(r1_16));
        ITEM_DIFF.put(STRIPPED_CRIMSON_HYPHAE, andNewer(r1_16));
        ITEM_DIFF.put(NETHER_GOLD_ORE, andNewer(r1_16));
        ITEM_DIFF.put(TWISTING_VINES, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BASALT, andNewer(r1_16));
        ITEM_DIFF.put(RESPAWN_ANCHOR, andNewer(r1_16));
        ITEM_DIFF.put(LODESTONE, andNewer(r1_16));
        ITEM_DIFF.put(CHAIN, andNewer(r1_16));
        ITEM_DIFF.put(SOUL_CAMPFIRE, andNewer(r1_16));
        ITEM_DIFF.put(BLACKSTONE, andNewer(r1_16));
        ITEM_DIFF.put(BLACKSTONE_STAIRS, andNewer(r1_16));
        ITEM_DIFF.put(BLACKSTONE_WALL, andNewer(r1_16));
        ITEM_DIFF.put(BLACKSTONE_SLAB, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_BRICKS, andNewer(r1_16));
        ITEM_DIFF.put(CRACKED_POLISHED_BLACKSTONE_BRICKS, andNewer(r1_16));
        ITEM_DIFF.put(CHISELED_POLISHED_BLACKSTONE, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_BRICK_SLAB, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_BRICK_STAIRS, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_BRICK_WALL, andNewer(r1_16));
        ITEM_DIFF.put(GILDED_BLACKSTONE, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_STAIRS, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_SLAB, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_PRESSURE_PLATE, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_BUTTON, andNewer(r1_16));
        ITEM_DIFF.put(POLISHED_BLACKSTONE_WALL, andNewer(r1_16));
        ITEM_DIFF.put(CHISELED_NETHER_BRICKS, andNewer(r1_16));
        ITEM_DIFF.put(CRACKED_NETHER_BRICKS, andNewer(r1_16));
        ITEM_DIFF.put(QUARTZ_BRICKS, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_SCRAP, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_INGOT, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_HELMET, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_CHESTPLATE, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_LEGGINGS, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_BOOTS, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_SWORD, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_SHOVEL, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_AXE, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_PICKAXE, andNewer(r1_16));
        ITEM_DIFF.put(NETHERITE_HOE, andNewer(r1_16));
        ITEM_DIFF.put(WARPED_FUNGUS_ON_A_STICK, andNewer(r1_16));
        ITEM_DIFF.put(MUSIC_DISC_PIGSTEP, andNewer(r1_16));
        ITEM_DIFF.put(PIGLIN_BANNER_PATTERN, andNewer(r1_16));
        ITEM_DIFF.put(HOGLIN_SPAWN_EGG, andNewer(r1_16));
        ITEM_DIFF.put(PIGLIN_SPAWN_EGG, andNewer(r1_16));
        ITEM_DIFF.put(PIGLIN_BRUTE_SPAWN_EGG, andNewer(r1_16));
        ITEM_DIFF.put(STRIDER_SPAWN_EGG, andNewer(r1_16));
        ITEM_DIFF.put(ZOGLIN_SPAWN_EGG, andNewer(r1_16));

        ITEM_DIFF.put(BEE_NEST, andNewer(r1_15));
        ITEM_DIFF.put(BEEHIVE, andNewer(r1_15));
        ITEM_DIFF.put(HONEY_BLOCK, andNewer(r1_15));
        ITEM_DIFF.put(HONEYCOMB_BLOCK, andNewer(r1_15));
        ITEM_DIFF.put(HONEYCOMB, andNewer(r1_15));
        ITEM_DIFF.put(HONEY_BOTTLE, andNewer(r1_15));
        ITEM_DIFF.put(BEE_SPAWN_EGG, andNewer(r1_15));

        ITEM_DIFF.put(CORNFLOWER, andNewer(r1_14));
        ITEM_DIFF.put(LILY_OF_THE_VALLEY, andNewer(r1_14));
        ITEM_DIFF.put(WITHER_ROSE, andNewer(r1_14));
        ITEM_DIFF.put(STONE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(CUT_SANDSTONE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(CUT_RED_SANDSTONE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(BRICK_WALL, andNewer(r1_14));
        ITEM_DIFF.put(PRISMARINE_WALL, andNewer(r1_14));
        ITEM_DIFF.put(RED_SANDSTONE_WALL, andNewer(r1_14));
        ITEM_DIFF.put(MOSSY_STONE_BRICK_WALL, andNewer(r1_14));
        ITEM_DIFF.put(GRANITE_WALL, andNewer(r1_14));
        ITEM_DIFF.put(STONE_BRICK_WALL, andNewer(r1_14));
        ITEM_DIFF.put(NETHER_BRICK_WALL, andNewer(r1_14));
        ITEM_DIFF.put(ANDESITE_WALL, andNewer(r1_14));
        ITEM_DIFF.put(RED_NETHER_BRICK_WALL, andNewer(r1_14));
        ITEM_DIFF.put(SANDSTONE_WALL, andNewer(r1_14));
        ITEM_DIFF.put(END_STONE_BRICK_WALL, andNewer(r1_14));
        ITEM_DIFF.put(DIORITE_WALL, andNewer(r1_14));
        ITEM_DIFF.put(POLISHED_GRANITE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(SMOOTH_RED_SANDSTONE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(MOSSY_STONE_BRICK_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(POLISHED_DIORITE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(MOSSY_COBBLESTONE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(END_STONE_BRICK_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(STONE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(SMOOTH_SANDSTONE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(SMOOTH_QUARTZ_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(GRANITE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(ANDESITE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(RED_NETHER_BRICK_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(POLISHED_ANDESITE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(DIORITE_STAIRS, andNewer(r1_14));
        ITEM_DIFF.put(POLISHED_GRANITE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(SMOOTH_RED_SANDSTONE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(MOSSY_STONE_BRICK_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(POLISHED_DIORITE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(MOSSY_COBBLESTONE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(END_STONE_BRICK_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(SMOOTH_SANDSTONE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(SMOOTH_QUARTZ_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(GRANITE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(ANDESITE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(RED_NETHER_BRICK_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(POLISHED_ANDESITE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(DIORITE_SLAB, andNewer(r1_14));
        ITEM_DIFF.put(SCAFFOLDING, andNewer(r1_14));
        ITEM_DIFF.put(JIGSAW, andNewer(r1_14));
        ITEM_DIFF.put(COMPOSTER, andNewer(r1_14));
        ITEM_DIFF.put(SPRUCE_SIGN, andNewer(r1_14));
        ITEM_DIFF.put(BIRCH_SIGN, andNewer(r1_14));
        ITEM_DIFF.put(JUNGLE_SIGN, andNewer(r1_14));
        ITEM_DIFF.put(ACACIA_SIGN, andNewer(r1_14));
        ITEM_DIFF.put(DARK_OAK_SIGN, andNewer(r1_14));
        ITEM_DIFF.put(BAMBOO, andNewer(r1_14));
        ITEM_DIFF.put(BLUE_DYE, andNewer(r1_14));
        ITEM_DIFF.put(BROWN_DYE, andNewer(r1_14));
        ITEM_DIFF.put(BLACK_DYE, andNewer(r1_14));
        ITEM_DIFF.put(WHITE_DYE, andNewer(r1_14));
        ITEM_DIFF.put(CAT_SPAWN_EGG, andNewer(r1_14));
        ITEM_DIFF.put(FOX_SPAWN_EGG, andNewer(r1_14));
        ITEM_DIFF.put(PANDA_SPAWN_EGG, andNewer(r1_14));
        ITEM_DIFF.put(PILLAGER_SPAWN_EGG, andNewer(r1_14));
        ITEM_DIFF.put(RAVAGER_SPAWN_EGG, andNewer(r1_14));
        ITEM_DIFF.put(TRADER_LLAMA_SPAWN_EGG, andNewer(r1_14));
        ITEM_DIFF.put(WANDERING_TRADER_SPAWN_EGG, andNewer(r1_14));
        ITEM_DIFF.put(LEATHER_HORSE_ARMOR, andNewer(r1_14));
        ITEM_DIFF.put(CROSSBOW, andNewer(r1_14));
        ITEM_DIFF.put(SUSPICIOUS_STEW, andNewer(r1_14));
        ITEM_DIFF.put(LOOM, andNewer(r1_14));
        ITEM_DIFF.put(FLOWER_BANNER_PATTERN, andNewer(r1_14));
        ITEM_DIFF.put(CREEPER_BANNER_PATTERN, andNewer(r1_14));
        ITEM_DIFF.put(SKULL_BANNER_PATTERN, andNewer(r1_14));
        ITEM_DIFF.put(MOJANG_BANNER_PATTERN, andNewer(r1_14));
        ITEM_DIFF.put(GLOBE_BANNER_PATTERN, andNewer(r1_14));
        ITEM_DIFF.put(BARREL, andNewer(r1_14));
        ITEM_DIFF.put(SMOKER, andNewer(r1_14));
        ITEM_DIFF.put(BLAST_FURNACE, andNewer(r1_14));
        ITEM_DIFF.put(CARTOGRAPHY_TABLE, andNewer(r1_14));
        ITEM_DIFF.put(FLETCHING_TABLE, andNewer(r1_14));
        ITEM_DIFF.put(GRINDSTONE, andNewer(r1_14));
        ITEM_DIFF.put(LECTERN, andNewer(r1_14));
        ITEM_DIFF.put(SMITHING_TABLE, andNewer(r1_14));
        ITEM_DIFF.put(STONECUTTER, andNewer(r1_14));
        ITEM_DIFF.put(BELL, andNewer(r1_14));
        ITEM_DIFF.put(LANTERN, andNewer(r1_14));
        ITEM_DIFF.put(SWEET_BERRIES, andNewer(r1_14));
        ITEM_DIFF.put(CAMPFIRE, andNewer(r1_14));

        ITEM_DIFF.put(DEAD_BRAIN_CORAL, andNewer(r1_13_1));
        ITEM_DIFF.put(DEAD_BUBBLE_CORAL, andNewer(r1_13_1));
        ITEM_DIFF.put(DEAD_FIRE_CORAL, andNewer(r1_13_1));
        ITEM_DIFF.put(DEAD_HORN_CORAL, andNewer(r1_13_1));
        ITEM_DIFF.put(DEAD_TUBE_CORAL, andNewer(r1_13_1));

        ITEM_DIFF.put(STRIPPED_OAK_LOG, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_SPRUCE_LOG, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_BIRCH_LOG, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_JUNGLE_LOG, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_ACACIA_LOG, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_DARK_OAK_LOG, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_OAK_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_SPRUCE_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_BIRCH_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_JUNGLE_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_ACACIA_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(STRIPPED_DARK_OAK_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(OAK_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(SPRUCE_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(BIRCH_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(JUNGLE_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(ACACIA_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(DARK_OAK_WOOD, andNewer(r1_13));
        ITEM_DIFF.put(SEAGRASS, andNewer(r1_13));
        ITEM_DIFF.put(SEA_PICKLE, andNewer(r1_13));
        ITEM_DIFF.put(PRISMARINE_SLAB, andNewer(r1_13));
        ITEM_DIFF.put(PRISMARINE_BRICK_SLAB, andNewer(r1_13));
        ITEM_DIFF.put(DARK_PRISMARINE_SLAB, andNewer(r1_13));
        ITEM_DIFF.put(SMOOTH_QUARTZ, andNewer(r1_13));
        ITEM_DIFF.put(SMOOTH_RED_SANDSTONE, andNewer(r1_13));
        ITEM_DIFF.put(SMOOTH_SANDSTONE, andNewer(r1_13));
        ITEM_DIFF.put(SMOOTH_STONE, andNewer(r1_13));
        ITEM_DIFF.put(SPRUCE_PRESSURE_PLATE, andNewer(r1_13));
        ITEM_DIFF.put(BIRCH_PRESSURE_PLATE, andNewer(r1_13));
        ITEM_DIFF.put(JUNGLE_PRESSURE_PLATE, andNewer(r1_13));
        ITEM_DIFF.put(ACACIA_PRESSURE_PLATE, andNewer(r1_13));
        ITEM_DIFF.put(DARK_OAK_PRESSURE_PLATE, andNewer(r1_13));
        ITEM_DIFF.put(PUMPKIN, andNewer(r1_13));
        ITEM_DIFF.put(SPRUCE_TRAPDOOR, andNewer(r1_13));
        ITEM_DIFF.put(BIRCH_TRAPDOOR, andNewer(r1_13));
        ITEM_DIFF.put(JUNGLE_TRAPDOOR, andNewer(r1_13));
        ITEM_DIFF.put(ACACIA_TRAPDOOR, andNewer(r1_13));
        ITEM_DIFF.put(DARK_OAK_TRAPDOOR, andNewer(r1_13));
        ITEM_DIFF.put(SPRUCE_BUTTON, andNewer(r1_13));
        ITEM_DIFF.put(BIRCH_BUTTON, andNewer(r1_13));
        ITEM_DIFF.put(JUNGLE_BUTTON, andNewer(r1_13));
        ITEM_DIFF.put(ACACIA_BUTTON, andNewer(r1_13));
        ITEM_DIFF.put(DARK_OAK_BUTTON, andNewer(r1_13));
        ITEM_DIFF.put(PRISMARINE_STAIRS, andNewer(r1_13));
        ITEM_DIFF.put(PRISMARINE_BRICK_STAIRS, andNewer(r1_13));
        ITEM_DIFF.put(DARK_PRISMARINE_STAIRS, andNewer(r1_13));
        ITEM_DIFF.put(SHULKER_BOX, andNewer(r1_13));
        ITEM_DIFF.put(TURTLE_EGG, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_TUBE_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_BRAIN_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_BUBBLE_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_FIRE_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_HORN_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(TUBE_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(BRAIN_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(BUBBLE_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(FIRE_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(HORN_CORAL_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(TUBE_CORAL, andNewer(r1_13));
        ITEM_DIFF.put(BRAIN_CORAL, andNewer(r1_13));
        ITEM_DIFF.put(BUBBLE_CORAL, andNewer(r1_13));
        ITEM_DIFF.put(FIRE_CORAL, andNewer(r1_13));
        ITEM_DIFF.put(HORN_CORAL, andNewer(r1_13));
        ITEM_DIFF.put(TUBE_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(BRAIN_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(BUBBLE_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(FIRE_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(HORN_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_TUBE_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_BRAIN_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_BUBBLE_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_FIRE_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(DEAD_HORN_CORAL_FAN, andNewer(r1_13));
        ITEM_DIFF.put(BLUE_ICE, andNewer(r1_13));
        ITEM_DIFF.put(CONDUIT, andNewer(r1_13));
        ITEM_DIFF.put(TURTLE_HELMET, andNewer(r1_13));
        ITEM_DIFF.put(SCUTE, andNewer(r1_13));
        ITEM_DIFF.put(PUFFERFISH_BUCKET, andNewer(r1_13));
        ITEM_DIFF.put(SALMON_BUCKET, andNewer(r1_13));
        ITEM_DIFF.put(COD_BUCKET, andNewer(r1_13));
        ITEM_DIFF.put(TROPICAL_FISH_BUCKET, andNewer(r1_13));
        ITEM_DIFF.put(KELP, andNewer(r1_13));
        ITEM_DIFF.put(DRIED_KELP_BLOCK, andNewer(r1_13));
        ITEM_DIFF.put(DRIED_KELP, andNewer(r1_13));
        ITEM_DIFF.put(COD_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(DOLPHIN_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(DROWNED_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(PHANTOM_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(PUFFERFISH_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(SALMON_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(TROPICAL_FISH_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(TURTLE_SPAWN_EGG, andNewer(r1_13));
        ITEM_DIFF.put(DEBUG_STICK, andNewer(r1_13));
        ITEM_DIFF.put(TRIDENT, andNewer(r1_13));
        ITEM_DIFF.put(PHANTOM_MEMBRANE, andNewer(r1_13));
        ITEM_DIFF.put(NAUTILUS_SHELL, andNewer(r1_13));
        ITEM_DIFF.put(HEART_OF_THE_SEA, andNewer(r1_13));

        ITEM_DIFF.put(KNOWLEDGE_BOOK, andNewer(r1_12));
        ITEM_DIFF.put(WHITE_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(ORANGE_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(MAGENTA_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_BLUE_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(YELLOW_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(LIME_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(PINK_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(GRAY_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_GRAY_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(CYAN_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(PURPLE_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(BLUE_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(BROWN_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(GREEN_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(RED_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(BLACK_GLAZED_TERRACOTTA, andNewer(r1_12));
        ITEM_DIFF.put(WHITE_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(ORANGE_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(MAGENTA_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_BLUE_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(YELLOW_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(LIME_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(PINK_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(GRAY_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_GRAY_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(CYAN_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(PURPLE_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(BLUE_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(BROWN_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(GREEN_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(RED_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(BLACK_CONCRETE, andNewer(r1_12));
        ITEM_DIFF.put(WHITE_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(ORANGE_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(MAGENTA_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_BLUE_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(YELLOW_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(LIME_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(PINK_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(GRAY_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_GRAY_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(CYAN_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(PURPLE_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(BLUE_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(BROWN_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(GREEN_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(RED_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(BLACK_CONCRETE_POWDER, andNewer(r1_12));
        ITEM_DIFF.put(WHITE_BED, andNewer(r1_12));
        ITEM_DIFF.put(ORANGE_BED, andNewer(r1_12));
        ITEM_DIFF.put(MAGENTA_BED, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_BLUE_BED, andNewer(r1_12));
        ITEM_DIFF.put(YELLOW_BED, andNewer(r1_12));
        ITEM_DIFF.put(LIME_BED, andNewer(r1_12));
        ITEM_DIFF.put(PINK_BED, andNewer(r1_12));
        ITEM_DIFF.put(GRAY_BED, andNewer(r1_12));
        ITEM_DIFF.put(LIGHT_GRAY_BED, andNewer(r1_12));
        ITEM_DIFF.put(CYAN_BED, andNewer(r1_12));
        ITEM_DIFF.put(PURPLE_BED, andNewer(r1_12));
        ITEM_DIFF.put(BLUE_BED, andNewer(r1_12));
        ITEM_DIFF.put(BROWN_BED, andNewer(r1_12));
        ITEM_DIFF.put(GREEN_BED, andNewer(r1_12));
        ITEM_DIFF.put(BLACK_BED, andNewer(r1_12));
        ITEM_DIFF.put(PARROT_SPAWN_EGG, andNewer(r1_12));

        ITEM_DIFF.put(IRON_NUGGET, andNewer(r1_11_1to1_11_2));

        ITEM_DIFF.put(OBSERVER, andNewer(r1_11));
        ITEM_DIFF.put(WHITE_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(ORANGE_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(MAGENTA_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(LIGHT_BLUE_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(YELLOW_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(LIME_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(PINK_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(GRAY_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(LIGHT_GRAY_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(CYAN_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(PURPLE_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(BLUE_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(BROWN_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(GREEN_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(RED_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(BLACK_SHULKER_BOX, andNewer(r1_11));
        ITEM_DIFF.put(TOTEM_OF_UNDYING, andNewer(r1_11));
        ITEM_DIFF.put(SHULKER_SHELL, andNewer(r1_11));
        ITEM_DIFF.put(VEX_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(POLAR_BEAR_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(STRAY_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(HUSK_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(ELDER_GUARDIAN_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(WITHER_SKELETON_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(DONKEY_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(MULE_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(SKELETON_HORSE_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(ZOMBIE_HORSE_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(ZOMBIE_VILLAGER_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(EVOKER_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(LLAMA_SPAWN_EGG, andNewer(r1_11));
        ITEM_DIFF.put(VINDICATOR_SPAWN_EGG, andNewer(r1_11));

        ITEM_DIFF.put(MAGMA_BLOCK, andNewer(r1_10));
        ITEM_DIFF.put(NETHER_WART_BLOCK, andNewer(r1_10));
        ITEM_DIFF.put(RED_NETHER_BRICKS, andNewer(r1_10));
        ITEM_DIFF.put(BONE_BLOCK, andNewer(r1_10));
        ITEM_DIFF.put(STRUCTURE_VOID, andNewer(r1_10));
        ITEM_DIFF.put(STRUCTURE_BLOCK, andNewer(r1_10));

        ITEM_DIFF.put(END_ROD, andNewer(r1_9));
        ITEM_DIFF.put(CHORUS_PLANT, andNewer(r1_9));
        ITEM_DIFF.put(CHORUS_FLOWER, andNewer(r1_9));
        ITEM_DIFF.put(PURPUR_BLOCK, andNewer(r1_9));
        ITEM_DIFF.put(PURPUR_PILLAR, andNewer(r1_9));
        ITEM_DIFF.put(PURPUR_STAIRS, andNewer(r1_9));
        ITEM_DIFF.put(PURPUR_SLAB, andNewer(r1_9));
        ITEM_DIFF.put(END_STONE_BRICKS, andNewer(r1_9));
        ITEM_DIFF.put(DIRT_PATH, andNewer(r1_9));
        ITEM_DIFF.put(REPEATING_COMMAND_BLOCK, andNewer(r1_9));
        ITEM_DIFF.put(CHAIN_COMMAND_BLOCK, andNewer(r1_9));
        ITEM_DIFF.put(END_CRYSTAL, andNewer(r1_9));
        ITEM_DIFF.put(CHORUS_FRUIT, andNewer(r1_9));
        ITEM_DIFF.put(POPPED_CHORUS_FRUIT, andNewer(r1_9));
        ITEM_DIFF.put(BEETROOT, andNewer(r1_9));
        ITEM_DIFF.put(BEETROOT_SEEDS, andNewer(r1_9));
        ITEM_DIFF.put(BEETROOT_SOUP, andNewer(r1_9));
        ITEM_DIFF.put(DRAGON_HEAD, andNewer(r1_9));
        ITEM_DIFF.put(DRAGON_BREATH, andNewer(r1_9));
        ITEM_DIFF.put(SPECTRAL_ARROW, andNewer(r1_9));
        ITEM_DIFF.put(TIPPED_ARROW, andNewer(r1_9));
        ITEM_DIFF.put(SPRUCE_BOAT, andNewer(r1_9));
        ITEM_DIFF.put(BIRCH_BOAT, andNewer(r1_9));
        ITEM_DIFF.put(JUNGLE_BOAT, andNewer(r1_9));
        ITEM_DIFF.put(ACACIA_BOAT, andNewer(r1_9));
        ITEM_DIFF.put(DARK_OAK_BOAT, andNewer(r1_9));
        ITEM_DIFF.put(LINGERING_POTION, andNewer(r1_9));
        ITEM_DIFF.put(SHIELD, andNewer(r1_9));
        ITEM_DIFF.put(ELYTRA, andNewer(r1_9));
        ITEM_DIFF.put(SHULKER_SPAWN_EGG, andNewer(r1_9));

        ITEM_DIFF.put(ACACIA_DOOR, andNewer(r1_8));
        ITEM_DIFF.put(ACACIA_FENCE, andNewer(r1_8));
        ITEM_DIFF.put(ACACIA_FENCE_GATE, andNewer(r1_8));
        ITEM_DIFF.put(ARMOR_STAND, andNewer(r1_8));
        ITEM_DIFF.put(WHITE_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(ORANGE_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(MAGENTA_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(LIGHT_BLUE_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(YELLOW_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(LIME_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(PINK_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(GRAY_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(LIGHT_GRAY_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(CYAN_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(PURPLE_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(BLUE_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(BROWN_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(GREEN_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(RED_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(BLACK_BANNER, andNewer(r1_8));
        ITEM_DIFF.put(BARRIER, andNewer(r1_8));
        ITEM_DIFF.put(BIRCH_DOOR, andNewer(r1_8));
        ITEM_DIFF.put(BIRCH_FENCE, andNewer(r1_8));
        ITEM_DIFF.put(BIRCH_FENCE_GATE, andNewer(r1_8));
        ITEM_DIFF.put(COOKED_MUTTON, andNewer(r1_8));
        ITEM_DIFF.put(COOKED_RABBIT, andNewer(r1_8));
        ITEM_DIFF.put(DARK_OAK_DOOR, andNewer(r1_8));
        ITEM_DIFF.put(DARK_OAK_FENCE, andNewer(r1_8));
        ITEM_DIFF.put(DARK_OAK_FENCE_GATE, andNewer(r1_8));
        ITEM_DIFF.put(IRON_TRAPDOOR, andNewer(r1_8));
        ITEM_DIFF.put(JUNGLE_DOOR, andNewer(r1_8));
        ITEM_DIFF.put(JUNGLE_FENCE, andNewer(r1_8));
        ITEM_DIFF.put(JUNGLE_FENCE_GATE, andNewer(r1_8));
        ITEM_DIFF.put(MUTTON, andNewer(r1_8));
        ITEM_DIFF.put(PRISMARINE, andNewer(r1_8));
        ITEM_DIFF.put(PRISMARINE_CRYSTALS, andNewer(r1_8));
        ITEM_DIFF.put(PRISMARINE_SHARD, andNewer(r1_8));
        ITEM_DIFF.put(RABBIT, andNewer(r1_8));
        ITEM_DIFF.put(RABBIT_FOOT, andNewer(r1_8));
        ITEM_DIFF.put(RABBIT_HIDE, andNewer(r1_8));
        ITEM_DIFF.put(RABBIT_STEW, andNewer(r1_8));
        ITEM_DIFF.put(RED_SANDSTONE, andNewer(r1_8));
        ITEM_DIFF.put(RED_SANDSTONE_STAIRS, andNewer(r1_8));
        ITEM_DIFF.put(CUT_RED_SANDSTONE, andNewer(r1_8));
        ITEM_DIFF.put(CHISELED_RED_SANDSTONE, andNewer(r1_8));
        ITEM_DIFF.put(RED_SANDSTONE_SLAB, andNewer(r1_8));
        ITEM_DIFF.put(SEA_LANTERN, andNewer(r1_8));
        ITEM_DIFF.put(SLIME_BLOCK, andNewer(r1_8));
        ITEM_DIFF.put(SPRUCE_DOOR, andNewer(r1_8));
        ITEM_DIFF.put(SPRUCE_FENCE, andNewer(r1_8));
        ITEM_DIFF.put(SPRUCE_FENCE_GATE, andNewer(r1_8));
        ITEM_DIFF.put(DIORITE, andNewer(r1_8));
        ITEM_DIFF.put(POLISHED_DIORITE, andNewer(r1_8));
        ITEM_DIFF.put(ANDESITE, andNewer(r1_8));
        ITEM_DIFF.put(POLISHED_ANDESITE, andNewer(r1_8));
        ITEM_DIFF.put(GRANITE, andNewer(r1_8));
        ITEM_DIFF.put(POLISHED_GRANITE, andNewer(r1_8));
        ITEM_DIFF.put(DARK_PRISMARINE, andNewer(r1_8));
        ITEM_DIFF.put(PRISMARINE_BRICKS, andNewer(r1_8));
        ITEM_DIFF.put(WET_SPONGE, andNewer(r1_8));
        ITEM_DIFF.put(COARSE_DIRT, andNewer(r1_8));
        ITEM_DIFF.put(RABBIT_SPAWN_EGG, andNewer(r1_8));
        ITEM_DIFF.put(GUARDIAN_SPAWN_EGG, andNewer(r1_8));
        ITEM_DIFF.put(ENDERMITE_SPAWN_EGG, andNewer(r1_8));

        ITEM_DIFF.put(WHITE_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ORANGE_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(MAGENTA_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LIGHT_BLUE_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(YELLOW_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LIME_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PINK_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(GRAY_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LIGHT_GRAY_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(CYAN_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PURPLE_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(BLUE_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(BROWN_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(GREEN_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(RED_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(BLACK_STAINED_GLASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(WHITE_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ORANGE_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(MAGENTA_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LIGHT_BLUE_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(YELLOW_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LIME_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PINK_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(GRAY_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LIGHT_GRAY_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(CYAN_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PURPLE_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(BLUE_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(BROWN_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(GREEN_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(RED_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(BLACK_STAINED_GLASS_PANE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ACACIA_LOG, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(DARK_OAK_LOG, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ACACIA_PLANKS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(DARK_OAK_PLANKS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ACACIA_SLAB, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(DARK_OAK_SLAB, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ACACIA_STAIRS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(DARK_OAK_STAIRS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ACACIA_SAPLING, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(DARK_OAK_SAPLING, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ACACIA_LEAVES, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(DARK_OAK_LEAVES, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PACKED_ICE, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PODZOL, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(RED_SAND, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(RED_TULIP, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ORANGE_TULIP, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(WHITE_TULIP, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PINK_TULIP, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(BLUE_ORCHID, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ALLIUM, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(AZURE_BLUET, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(OXEYE_DAISY, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(SUNFLOWER, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PEONY, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(ROSE_BUSH, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LILAC, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(TALL_GRASS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(LARGE_FERN, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(INFESTED_MOSSY_STONE_BRICKS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(INFESTED_CRACKED_STONE_BRICKS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(INFESTED_CHISELED_STONE_BRICKS, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(PUFFERFISH, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(SALMON, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(COOKED_SALMON, andNewer(r1_7_2tor1_7_5));
        ITEM_DIFF.put(TROPICAL_FISH, andNewer(r1_7_2tor1_7_5));

        ITEM_DIFF.put(HAY_BLOCK, andNewer(r1_6_1));
        ITEM_DIFF.put(WHITE_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(ORANGE_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(MAGENTA_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(LIGHT_BLUE_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(YELLOW_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(LIME_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(PINK_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(GRAY_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(LIGHT_GRAY_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(CYAN_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(PURPLE_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(BLUE_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(BROWN_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(GREEN_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(RED_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(BLACK_CARPET, andNewer(r1_6_1));
        ITEM_DIFF.put(TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(WHITE_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(ORANGE_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(MAGENTA_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(LIGHT_BLUE_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(YELLOW_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(LIME_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(PINK_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(GRAY_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(LIGHT_GRAY_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(CYAN_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(PURPLE_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(BLUE_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(BROWN_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(GREEN_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(RED_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(BLACK_TERRACOTTA, andNewer(r1_6_1));
        ITEM_DIFF.put(COAL_BLOCK, andNewer(r1_6_1));
        ITEM_DIFF.put(IRON_HORSE_ARMOR, andNewer(r1_6_1));
        ITEM_DIFF.put(GOLDEN_HORSE_ARMOR, andNewer(r1_6_1));
        ITEM_DIFF.put(DIAMOND_HORSE_ARMOR, andNewer(r1_6_1));
        ITEM_DIFF.put(HORSE_SPAWN_EGG, andNewer(r1_6_1));
        ITEM_DIFF.put(LEAD, andNewer(r1_6_1));
        ITEM_DIFF.put(NAME_TAG, andNewer(r1_6_1));

        ITEM_DIFF.put(ACTIVATOR_RAIL, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(REDSTONE_BLOCK, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(DAYLIGHT_DETECTOR, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(DROPPER, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(HOPPER, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(COMPARATOR, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(TRAPPED_CHEST, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(LIGHT_WEIGHTED_PRESSURE_PLATE, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(HEAVY_WEIGHTED_PRESSURE_PLATE, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(NETHER_QUARTZ_ORE, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(QUARTZ_BLOCK, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(QUARTZ_SLAB, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(QUARTZ_STAIRS, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(CHISELED_QUARTZ_BLOCK, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(QUARTZ_PILLAR, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(NETHER_BRICK, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(QUARTZ, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(TNT_MINECART, andNewer(r1_5tor1_5_1));
        ITEM_DIFF.put(HOPPER_MINECART, andNewer(r1_5tor1_5_1));

        ITEM_DIFF.put(NETHER_BRICK_SLAB, andNewer(r1_4_6tor1_4_7));
        ITEM_DIFF.put(ENCHANTED_BOOK, andNewer(r1_4_6tor1_4_7));
        ITEM_DIFF.put(FIREWORK_STAR, andNewer(r1_4_6tor1_4_7));
        ITEM_DIFF.put(FIREWORK_ROCKET, andNewer(r1_4_6tor1_4_7));

        ITEM_DIFF.put(MUSIC_DISC_WAIT, andNewer(r1_4_4tor1_4_5));

        ITEM_DIFF.put(COMMAND_BLOCK, andNewer(r1_4_2));
        ITEM_DIFF.put(COMMAND_BLOCK_MINECART, andNewer(r1_4_2));
        ITEM_DIFF.put(BEACON, andNewer(r1_4_2));
        ITEM_DIFF.put(ANVIL, andNewer(r1_4_2));
        ITEM_DIFF.put(CHIPPED_ANVIL, andNewer(r1_4_2));
        ITEM_DIFF.put(DAMAGED_ANVIL, andNewer(r1_4_2));
        ITEM_DIFF.put(FLOWER_POT, andNewer(r1_4_2));
        ITEM_DIFF.put(COBBLESTONE_WALL, andNewer(r1_4_2));
        ITEM_DIFF.put(MOSSY_COBBLESTONE_WALL, andNewer(r1_4_2));
        ITEM_DIFF.put(CREEPER_HEAD, andNewer(r1_4_2));
        ITEM_DIFF.put(ZOMBIE_HEAD, andNewer(r1_4_2));
        ITEM_DIFF.put(SKELETON_SKULL, andNewer(r1_4_2));
        ITEM_DIFF.put(WITHER_SKELETON_SKULL, andNewer(r1_4_2));
        ITEM_DIFF.put(PLAYER_HEAD, andNewer(r1_4_2));
        ITEM_DIFF.put(OAK_BUTTON, andNewer(r1_4_2));
        ITEM_DIFF.put(POTATO, andNewer(r1_4_2));
        ITEM_DIFF.put(POISONOUS_POTATO, andNewer(r1_4_2));
        ITEM_DIFF.put(BAKED_POTATO, andNewer(r1_4_2));
        ITEM_DIFF.put(CARROT, andNewer(r1_4_2));
        ITEM_DIFF.put(GOLDEN_CARROT, andNewer(r1_4_2));
        ITEM_DIFF.put(CARROT_ON_A_STICK, andNewer(r1_4_2));
        ITEM_DIFF.put(PUMPKIN_PIE, andNewer(r1_4_2));
        ITEM_DIFF.put(NETHER_STAR, andNewer(r1_4_2));
        ITEM_DIFF.put(BAT_SPAWN_EGG, andNewer(r1_4_2));
        ITEM_DIFF.put(WITCH_SPAWN_EGG, andNewer(r1_4_2));
        ITEM_DIFF.put(ITEM_FRAME, andNewer(r1_4_2));

        ITEM_DIFF.put(EMERALD_ORE, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(EMERALD_BLOCK, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(EMERALD, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(ENDER_CHEST, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(TRIPWIRE_HOOK, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(SANDSTONE_STAIRS, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(WRITABLE_BOOK, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(WRITTEN_BOOK, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(ENCHANTED_GOLDEN_APPLE, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(OAK_SLAB, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(SPRUCE_SLAB, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(BIRCH_SLAB, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(JUNGLE_SLAB, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(SPRUCE_STAIRS, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(BIRCH_STAIRS, andNewer(r1_3_1tor1_3_2));
        ITEM_DIFF.put(JUNGLE_STAIRS, andNewer(r1_3_1tor1_3_2));

        ITEM_DIFF.put(BIRCH_PLANKS, andNewer(r1_2_4tor1_2_5));
        ITEM_DIFF.put(SPRUCE_PLANKS, andNewer(r1_2_4tor1_2_5));
        ITEM_DIFF.put(JUNGLE_PLANKS, andNewer(r1_2_4tor1_2_5));
        ITEM_DIFF.put(CHISELED_SANDSTONE, andNewer(r1_2_4tor1_2_5));
        ITEM_DIFF.put(CUT_SANDSTONE, andNewer(r1_2_4tor1_2_5));

        ITEM_DIFF.put(CHISELED_STONE_BRICKS, andNewer(r1_2_1tor1_2_3));
        ITEM_DIFF.put(REDSTONE_LAMP, andNewer(r1_2_1tor1_2_3));
        ITEM_DIFF.put(JUNGLE_LOG, andNewer(r1_2_1tor1_2_3));
        ITEM_DIFF.put(JUNGLE_LEAVES, andNewer(r1_2_1tor1_2_3));
        ITEM_DIFF.put(JUNGLE_SAPLING, andNewer(r1_2_1tor1_2_3));
        ITEM_DIFF.put(EXPERIENCE_BOTTLE, andNewer(r1_2_1tor1_2_3));
        ITEM_DIFF.put(FIRE_CHARGE, andNewer(r1_2_1tor1_2_3));
        ITEM_DIFF.put(OCELOT_SPAWN_EGG, andNewer(r1_2_1tor1_2_3));

        ITEM_DIFF.put(BLAZE_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(CAVE_SPIDER_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(CHICKEN_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(COW_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(CREEPER_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(ENDERMAN_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(GHAST_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(MAGMA_CUBE_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(MOOSHROOM_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(PIG_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(SHEEP_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(SILVERFISH_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(SKELETON_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(SLIME_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(SPIDER_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(SQUID_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(VILLAGER_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(WOLF_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(ZOMBIE_SPAWN_EGG, andNewer(r1_1));
        ITEM_DIFF.put(ZOMBIFIED_PIGLIN_SPAWN_EGG, andNewer(r1_1));

        ITEM_DIFF.put(BREWING_STAND, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(CAULDRON, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(DRAGON_EGG, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(ENCHANTING_TABLE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(END_PORTAL_FRAME, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(END_STONE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(LILY_PAD, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MYCELIUM, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(NETHER_BRICKS, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(NETHER_BRICK_FENCE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(NETHER_BRICK_STAIRS, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(NETHER_WART, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(BLAZE_POWDER, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(BLAZE_ROD, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(ENDER_EYE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(SPIDER_EYE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(FERMENTED_SPIDER_EYE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(GHAST_TEAR, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(GLASS_BOTTLE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(GLISTERING_MELON_SLICE, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(GOLD_NUGGET, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MAGMA_CREAM, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_13, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_CAT, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_BLOCKS, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_CHIRP, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_FAR, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_MALL, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_MELLOHI, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_STAL, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_STRAD, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_WARD, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(MUSIC_DISC_11, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(POTION, andNewer(r1_0_0tor1_0_1));
        ITEM_DIFF.put(SPLASH_POTION, andNewer(r1_0_0tor1_0_1));

        ITEM_DIFF.put(STONE_BRICKS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(MOSSY_STONE_BRICKS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(CRACKED_STONE_BRICKS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(INFESTED_STONE, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(INFESTED_STONE_BRICKS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(INFESTED_COBBLESTONE, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(BRICK_SLAB, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(STONE_BRICK_SLAB, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(MUSHROOM_STEM, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(BROWN_MUSHROOM_BLOCK, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(RED_MUSHROOM_BLOCK, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(MELON, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(VINE, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(IRON_BARS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(GLASS_PANE, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(OAK_FENCE_GATE, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(BRICK_STAIRS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(STONE_BRICK_STAIRS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(CHICKEN, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(COOKED_CHICKEN, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(BEEF, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(COOKED_BEEF, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(ENDER_PEARL, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(MELON_SEEDS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(MELON_SLICE, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(PUMPKIN_SEEDS, andNewer(b1_8tob1_8_1));
        ITEM_DIFF.put(ROTTEN_FLESH, andNewer(b1_8tob1_8_1));

        ITEM_DIFF.put(PISTON, andNewer(b1_7tob1_7_3));
        ITEM_DIFF.put(STICKY_PISTON, andNewer(b1_7tob1_7_3));
        ITEM_DIFF.put(SHEARS, andNewer(b1_7tob1_7_3));

        ITEM_DIFF.put(DEAD_BUSH, andNewer(b1_6tob1_6_6));
        ITEM_DIFF.put(SHORT_GRASS, andNewer(b1_6tob1_6_6));
        ITEM_DIFF.put(FERN, andNewer(b1_6tob1_6_6));
        ITEM_DIFF.put(OAK_TRAPDOOR, andNewer(b1_6tob1_6_6));
        ITEM_DIFF.put(MAP, andNewer(b1_6tob1_6_6));
        ITEM_DIFF.put(FILLED_MAP, andNewer(b1_6tob1_6_6));

        ITEM_DIFF.put(BIRCH_SAPLING, andNewer(b1_5tob1_5_2));
        ITEM_DIFF.put(COBWEB, andNewer(b1_5tob1_5_2));
        ITEM_DIFF.put(DETECTOR_RAIL, andNewer(b1_5tob1_5_2));
        ITEM_DIFF.put(POWERED_RAIL, andNewer(b1_5tob1_5_2));
        ITEM_DIFF.put(SPRUCE_SAPLING, andNewer(b1_5tob1_5_2));

        ITEM_DIFF.put(COOKIE, andNewer(b1_4tob1_4_1));

        ITEM_DIFF.put(RED_BED, andNewer(b1_3tob1_3_1));
        ITEM_DIFF.put(REPEATER, andNewer(b1_3tob1_3_1));
        ITEM_DIFF.put(COBBLESTONE_SLAB, andNewer(b1_3tob1_3_1));
        ITEM_DIFF.put(PETRIFIED_OAK_SLAB, andNewer(b1_3tob1_3_1));
        ITEM_DIFF.put(SANDSTONE_SLAB, andNewer(b1_3tob1_3_1));
        ITEM_DIFF.put(PAINTING, andNewer(b1_3tob1_3_1));

        ITEM_DIFF.put(CAKE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(DISPENSER, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(LAPIS_LAZULI, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(LAPIS_ORE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(LAPIS_BLOCK, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(NOTE_BLOCK, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(SANDSTONE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(SPRUCE_LOG, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(BIRCH_LOG, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(SPRUCE_LEAVES, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(BIRCH_LEAVES, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(BONE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(BONE_MEAL, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(COCOA_BEANS, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(INK_SAC, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(CHARCOAL, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(SUGAR, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(PURPLE_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(CYAN_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(LIGHT_GRAY_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(GRAY_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(PINK_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(LIME_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(YELLOW_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(LIGHT_BLUE_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(MAGENTA_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(ORANGE_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(RED_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(GREEN_DYE, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(ORANGE_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(MAGENTA_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(LIGHT_BLUE_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(YELLOW_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(LIME_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(PINK_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(GRAY_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(LIGHT_GRAY_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(CYAN_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(PURPLE_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(BLUE_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(BROWN_WOOL, andNewer(b1_2_0tob1_2_2));
        ITEM_DIFF.put(GREEN_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(RED_WOOL, andNewer(b1_2_0tob1_2_2).add(of(c0_0_20ac0_27, c0_28toc0_30)));
        ITEM_DIFF.put(BLACK_WOOL, andNewer(b1_2_0tob1_2_2));

        ITEM_DIFF.put(JACK_O_LANTERN, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(GLOWSTONE, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(CLOCK, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(NETHERRACK, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(COD, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(COOKED_COD, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(SOUL_SAND, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(CARVED_PUMPKIN, andNewer(a1_2_0toa1_2_1_1));
        ITEM_DIFF.put(GLOWSTONE_DUST, andNewer(a1_2_0toa1_2_1_1));

        ITEM_DIFF.put(FISHING_ROD, andNewer(a1_1_0toa1_1_2_1));
        ITEM_DIFF.put(COMPASS, andNewer(a1_1_0toa1_1_2_1));

        ITEM_DIFF.put(GRASS_BLOCK, andNewer(a1_0_15).add(single(c0_30cpe)));
        ITEM_DIFF.put(BEDROCK, andNewer(a1_0_15).add(single(c0_30cpe)));
        ITEM_DIFF.put(WATER_BUCKET, andNewer(a1_0_15).add(single(c0_30cpe)));
        ITEM_DIFF.put(LAVA_BUCKET, andNewer(a1_0_15).add(single(c0_30cpe)));

        ITEM_DIFF.put(WOODEN_AXE, andNewer(a1_0_15));
        ITEM_DIFF.put(WOODEN_HOE, andNewer(a1_0_15));
        ITEM_DIFF.put(WOODEN_PICKAXE, andNewer(a1_0_15));
        ITEM_DIFF.put(WOODEN_SHOVEL, andNewer(a1_0_15));
        ITEM_DIFF.put(WOODEN_SWORD, andNewer(a1_0_15));
        ITEM_DIFF.put(STONE_AXE, andNewer(a1_0_15));
        ITEM_DIFF.put(STONE_HOE, andNewer(a1_0_15));
        ITEM_DIFF.put(STONE_PICKAXE, andNewer(a1_0_15));
        ITEM_DIFF.put(STONE_SHOVEL, andNewer(a1_0_15));
        ITEM_DIFF.put(STONE_SWORD, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_AXE, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_HOE, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_PICKAXE, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_SHOVEL, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_SWORD, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_AXE, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_HOE, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_PICKAXE, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_SHOVEL, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_SWORD, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_AXE, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_HOE, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_PICKAXE, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_SHOVEL, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_SWORD, andNewer(a1_0_15));
        ITEM_DIFF.put(LEATHER_HELMET, andNewer(a1_0_15));
        ITEM_DIFF.put(LEATHER_CHESTPLATE, andNewer(a1_0_15));
        ITEM_DIFF.put(LEATHER_LEGGINGS, andNewer(a1_0_15));
        ITEM_DIFF.put(LEATHER_BOOTS, andNewer(a1_0_15));
        ITEM_DIFF.put(CHAINMAIL_HELMET, andNewer(a1_0_15));
        ITEM_DIFF.put(CHAINMAIL_CHESTPLATE, andNewer(a1_0_15));
        ITEM_DIFF.put(CHAINMAIL_LEGGINGS, andNewer(a1_0_15));
        ITEM_DIFF.put(CHAINMAIL_BOOTS, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_HELMET, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_CHESTPLATE, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_LEGGINGS, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_BOOTS, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_HELMET, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_CHESTPLATE, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_LEGGINGS, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_BOOTS, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_HELMET, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_CHESTPLATE, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_LEGGINGS, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_BOOTS, andNewer(a1_0_15));
        ITEM_DIFF.put(BUCKET, andNewer(a1_0_15));
        ITEM_DIFF.put(MILK_BUCKET, andNewer(a1_0_15));
        ITEM_DIFF.put(FLINT_AND_STEEL, andNewer(a1_0_15));
        ITEM_DIFF.put(SADDLE, andNewer(a1_0_15));
        ITEM_DIFF.put(OAK_BOAT, andNewer(a1_0_15));
        ITEM_DIFF.put(RAIL, andNewer(a1_0_15));
        ITEM_DIFF.put(MINECART, andNewer(a1_0_15));
        ITEM_DIFF.put(CHEST_MINECART, andNewer(a1_0_15));
        ITEM_DIFF.put(FURNACE_MINECART, andNewer(a1_0_15));
        ITEM_DIFF.put(SNOWBALL, andNewer(a1_0_15));
        ITEM_DIFF.put(EGG, andNewer(a1_0_15));
        ITEM_DIFF.put(BOW, andNewer(a1_0_15));
        ITEM_DIFF.put(ARROW, andNewer(a1_0_15));
        ITEM_DIFF.put(APPLE, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLDEN_APPLE, andNewer(a1_0_15));
        ITEM_DIFF.put(PORKCHOP, andNewer(a1_0_15));
        ITEM_DIFF.put(COOKED_PORKCHOP, andNewer(a1_0_15));
        ITEM_DIFF.put(BREAD, andNewer(a1_0_15));
        ITEM_DIFF.put(MUSHROOM_STEW, andNewer(a1_0_15));
        ITEM_DIFF.put(COAL, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_INGOT, andNewer(a1_0_15));
        ITEM_DIFF.put(GOLD_INGOT, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND, andNewer(a1_0_15));
        ITEM_DIFF.put(STICK, andNewer(a1_0_15));
        ITEM_DIFF.put(FLINT, andNewer(a1_0_15));
        ITEM_DIFF.put(WHEAT, andNewer(a1_0_15));
        ITEM_DIFF.put(STRING, andNewer(a1_0_15));
        ITEM_DIFF.put(FEATHER, andNewer(a1_0_15));
        ITEM_DIFF.put(LEATHER, andNewer(a1_0_15));
        ITEM_DIFF.put(SLIME_BALL, andNewer(a1_0_15));
        ITEM_DIFF.put(CLAY, andNewer(a1_0_15));
        ITEM_DIFF.put(CLAY_BALL, andNewer(a1_0_15));
        ITEM_DIFF.put(BOWL, andNewer(a1_0_15));
        ITEM_DIFF.put(BRICK, andNewer(a1_0_15));
        ITEM_DIFF.put(PAPER, andNewer(a1_0_15));
        ITEM_DIFF.put(BOOK, andNewer(a1_0_15));
        ITEM_DIFF.put(REDSTONE, andNewer(a1_0_15));
        ITEM_DIFF.put(GUNPOWDER, andNewer(a1_0_15));
        ITEM_DIFF.put(REDSTONE_TORCH, andNewer(a1_0_15));
        ITEM_DIFF.put(LEVER, andNewer(a1_0_15));
        ITEM_DIFF.put(STONE_BUTTON, andNewer(a1_0_15));
        ITEM_DIFF.put(OAK_PRESSURE_PLATE, andNewer(a1_0_15));
        ITEM_DIFF.put(STONE_PRESSURE_PLATE, andNewer(a1_0_15));
        ITEM_DIFF.put(CHEST, andNewer(a1_0_15));
        ITEM_DIFF.put(FURNACE, andNewer(a1_0_15));
        ITEM_DIFF.put(REDSTONE_ORE, andNewer(a1_0_15));
        ITEM_DIFF.put(JUKEBOX, andNewer(a1_0_15));
        ITEM_DIFF.put(OAK_DOOR, andNewer(a1_0_15));
        ITEM_DIFF.put(IRON_DOOR, andNewer(a1_0_15));
        ITEM_DIFF.put(OAK_SIGN, andNewer(a1_0_15));
        ITEM_DIFF.put(LADDER, andNewer(a1_0_15));
        ITEM_DIFF.put(TORCH, andNewer(a1_0_15));
        ITEM_DIFF.put(CRAFTING_TABLE, andNewer(a1_0_15));
        ITEM_DIFF.put(CACTUS, andNewer(a1_0_15));
        ITEM_DIFF.put(SUGAR_CANE, andNewer(a1_0_15));
        ITEM_DIFF.put(WHEAT_SEEDS, andNewer(a1_0_15));
        ITEM_DIFF.put(FARMLAND, andNewer(a1_0_15));
        ITEM_DIFF.put(SNOW, andNewer(a1_0_15));
        ITEM_DIFF.put(SNOW_BLOCK, andNewer(a1_0_15));
        ITEM_DIFF.put(ICE, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_ORE, andNewer(a1_0_15));
        ITEM_DIFF.put(DIAMOND_BLOCK, andNewer(a1_0_15));
        ITEM_DIFF.put(OAK_FENCE, andNewer(a1_0_15));
        ITEM_DIFF.put(OAK_STAIRS, andNewer(a1_0_15));
        ITEM_DIFF.put(COBBLESTONE_STAIRS, andNewer(a1_0_15));

        ITEM_DIFF.put(OBSIDIAN, andNewer(c0_30cpe));
        ITEM_DIFF.put(TNT, andNewer(c0_30cpe));
        ITEM_DIFF.put(BOOKSHELF, andNewer(c0_30cpe));
        ITEM_DIFF.put(IRON_BLOCK, andNewer(c0_30cpe));
        ITEM_DIFF.put(BRICKS, andNewer(c0_30cpe));
        ITEM_DIFF.put(COAL_ORE, andNewer(c0_30cpe));
        ITEM_DIFF.put(GOLD_ORE, andNewer(c0_30cpe));
        ITEM_DIFF.put(IRON_ORE, andNewer(c0_30cpe));
        ITEM_DIFF.put(MOSSY_COBBLESTONE, andNewer(c0_30cpe));
        ITEM_DIFF.put(SMOOTH_STONE_SLAB, andNewer(c0_30cpe));

        ITEM_DIFF.put(DANDELION, andNewer(c0_0_20ac0_27));
        ITEM_DIFF.put(POPPY, andNewer(c0_0_20ac0_27));
        ITEM_DIFF.put(BROWN_MUSHROOM, andNewer(c0_0_20ac0_27));
        ITEM_DIFF.put(RED_MUSHROOM, andNewer(c0_0_20ac0_27));
        ITEM_DIFF.put(GOLD_BLOCK, andNewer(c0_0_20ac0_27));
        ITEM_DIFF.put(WHITE_WOOL, andNewer(c0_0_20ac0_27));

        ITEM_DIFF.put(SPONGE, andNewer(c0_0_19a_06));
        ITEM_DIFF.put(GLASS, andNewer(c0_0_19a_06));

        ITEM_DIFF.put(STONE, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(DIRT, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(COBBLESTONE, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(OAK_PLANKS, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(OAK_SAPLING, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(OAK_LOG, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(OAK_LEAVES, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(SAND, andNewer(c0_0_15a_1));
        ITEM_DIFF.put(GRAVEL, andNewer(c0_0_15a_1));

        EXTENDED_CLASSIC_ITEMS.add(COBBLESTONE_SLAB);
        EXTENDED_CLASSIC_ITEMS.add(DEAD_BUSH);
        EXTENDED_CLASSIC_ITEMS.add(SANDSTONE);
        EXTENDED_CLASSIC_ITEMS.add(SNOW);
        EXTENDED_CLASSIC_ITEMS.add(TORCH);
        EXTENDED_CLASSIC_ITEMS.add(BROWN_WOOL);
        EXTENDED_CLASSIC_ITEMS.add(ICE);
        EXTENDED_CLASSIC_ITEMS.add(CHISELED_QUARTZ_BLOCK);
        EXTENDED_CLASSIC_ITEMS.add(NETHER_QUARTZ_ORE);
        EXTENDED_CLASSIC_ITEMS.add(QUARTZ_PILLAR);
        EXTENDED_CLASSIC_ITEMS.add(JUKEBOX);
        EXTENDED_CLASSIC_ITEMS.add(STONE_BRICKS);

        // https://minecraft.gamepedia.com/Java_Edition_version_history
    }

    /**
     * @param item The item to check
     * @return true if the item exists in the current version, false otherwise, this will also check for CPE items (CustomBlocks V1 extension)
     */
    public static boolean keepItem(final Item item) {
        if (ProtocolHack.getTargetVersion().equals(c0_30cpe)) {
            final ClientPlayNetworkHandler handler = MinecraftClient.getInstance().getNetworkHandler();
            if (handler == null) {
                // Don't drop any items if the connection is not established yet
                return true;
            }
            final ExtensionProtocolMetadataStorage extensionProtocol = ((IClientConnection) handler.getConnection()).viaFabricPlus$getUserConnection().get(ExtensionProtocolMetadataStorage.class);
            if (extensionProtocol == null) { // Should never happen
                return false;
            }
            if (extensionProtocol.hasServerExtension(ClassicProtocolExtension.CUSTOM_BLOCKS, 1) && EXTENDED_CLASSIC_ITEMS.contains(item)) {
                return true;
            }
        }

        return contains(item, ProtocolHack.getTargetVersion());
    }

    /**
     * @param item    The item to check
     * @param version The version to check
     * @return true if the item is present in the version, false otherwise
     */
    public static boolean contains(final Item item, final VersionEnum version) {
        return !ITEM_DIFF.containsKey(item) || ITEM_DIFF.get(item).contains(version);
    }

}
